from ctypes import c_int, c_uint, c_uint8, c_void_p, c_char_p, c_bool, POINTER, CFUNCTYPE

import json

from ..lib import lib

from ..types.string import String

from .content import ContentType
from .headers import http_header
from .query import http_query_pairs_get_value

RequestMethod = c_int

REQUEST_METHOD_DELETE = 0
REQUEST_METHOD_GET = 1
REQUEST_METHOD_HEAD = 2
REQUEST_METHOD_POST = 3
REQUEST_METHOD_PUT = 4
REQUEST_METHOD_CONNECT = 5
REQUEST_METHOD_OPTIONS = 6
REQUEST_METHOD_TRACE = 7

REQUEST_METHOD_UNDEFINED = 8

RequestDeleteDecoded = CFUNCTYPE (None, c_void_p)
RequestDeleteCustom = CFUNCTYPE (None, c_void_p)

# getters
http_request_get_method = lib.http_request_get_method
http_request_get_method.argtypes = [c_void_p]
http_request_get_method.restype = RequestMethod

http_request_get_url = lib.http_request_get_url
http_request_get_url.argtypes = [c_void_p]
http_request_get_url.restype = POINTER (String)

http_request_get_query = lib.http_request_get_query
http_request_get_query.argtypes = [c_void_p]
http_request_get_query.restype = POINTER (String)

http_request_get_query_params = lib.http_request_get_query_params
http_request_get_query_params.argtypes = [c_void_p]
http_request_get_query_params.restype = c_void_p

http_request_get_n_params = lib.http_request_get_n_params
http_request_get_n_params.argtypes = [c_void_p]
http_request_get_n_params.restype = c_uint

http_request_get_param_at_idx = lib.http_request_get_param_at_idx
http_request_get_param_at_idx.argtypes = [c_void_p, c_uint]
http_request_get_param_at_idx.restype = POINTER (String)

http_request_get_header = lib.http_request_get_header
http_request_get_header.argtypes = [c_void_p, http_header]
http_request_get_header.restype = POINTER (String)

http_request_get_content_type = lib.http_request_get_content_type
http_request_get_content_type.argtypes = [c_void_p]
http_request_get_content_type.restype = ContentType

http_request_get_content_type_string = lib.http_request_get_content_type_string
http_request_get_content_type_string.argtypes = [c_void_p]
http_request_get_content_type_string.restype = POINTER (String)

http_request_content_type_is_json = lib.http_request_content_type_is_json
http_request_content_type_is_json.argtypes = [c_void_p]
http_request_content_type_is_json.restype = c_bool

http_request_get_decoded_data = lib.http_request_get_decoded_data
http_request_get_decoded_data.argtypes = [c_void_p]
http_request_get_decoded_data.restype = c_void_p

http_request_set_decoded_data = lib.http_request_set_decoded_data
http_request_set_decoded_data.argtypes = [c_void_p, c_void_p]

http_request_set_delete_decoded_data = lib.http_request_set_delete_decoded_data
http_request_set_delete_decoded_data.argtypes = [c_void_p, RequestDeleteDecoded]

http_request_set_default_delete_decoded_data = lib.http_request_set_default_delete_decoded_data
http_request_set_default_delete_decoded_data.argtypes = [c_void_p]

http_request_get_custom_data = lib.http_request_get_custom_data
http_request_get_custom_data.argtypes = [c_void_p]
http_request_get_custom_data.restype = c_void_p

http_request_set_custom_data = lib.http_request_set_custom_data
http_request_set_custom_data.argtypes = [c_void_p, c_void_p]

http_request_set_delete_custom_data = lib.http_request_set_delete_custom_data
http_request_set_delete_custom_data.argtypes = [c_void_p, RequestDeleteCustom]

http_request_set_default_delete_custom_data = lib.http_request_set_default_delete_custom_data
http_request_set_default_delete_custom_data.argtypes = [c_void_p]

http_request_get_body = lib.http_request_get_body
http_request_get_body.argtypes = [c_void_p]
http_request_get_body.restype = POINTER (String)

http_request_get_current_mpart = lib.http_request_get_current_mpart
http_request_get_current_mpart.argtypes = [c_void_p]
http_request_get_current_mpart.restype = c_void_p

http_request_get_n_files = lib.http_request_get_n_files
http_request_get_n_files.argtypes = [c_void_p]
http_request_get_n_files.restype = c_uint8

http_request_get_n_values = lib.http_request_get_n_values
http_request_get_n_values.argtypes = [c_void_p]
http_request_get_n_values.restype = c_uint8

http_request_get_dirname_len = lib.http_request_get_dirname_len
http_request_get_dirname_len.argtypes = [c_void_p]
http_request_get_dirname_len.restype = c_int

http_request_get_dirname = lib.http_request_get_dirname
http_request_get_dirname.argtypes = [c_void_p]
http_request_get_dirname.restype = c_char_p

http_request_set_dirname = lib.http_request_set_dirname

http_request_get_body_values = lib.http_request_get_body_values
http_request_get_body_values.argtypes = [c_void_p]
http_request_get_body_values.restype = c_void_p

def http_request_get_query_value (values, query_name):
	"""
	Function to get a query param from request
	# Parameters
	------------
	### values: DoubleList <KeyValuePair>
		key-value pairs parsed from x-www-form-urlencoded data or query params
	### query_name: string
		the key used to find a matching value
	"""
	value = http_query_pairs_get_value (values, query_name.encode ("utf-8"))
	return value.contents.str.decode ("utf-8")

def http_request_get_body_json (request):
	"""
	Function to get body in a dictionary
	# Parameters
	------------
	### request: HttpRequest
		reference to a HTTP request instance
	"""
	body_str = http_request_get_body (request)
	body = json.loads (body_str.contents.str.decode ("utf-8"))
	return body

# headers
http_request_headers_print = lib.http_request_headers_print
http_request_headers_print.argtypes = [c_void_p]

# query
http_request_query_params_print = lib.http_request_query_params_print
http_request_query_params_print.argtypes = [c_void_p]

http_request_query_params_get_value = lib.http_request_query_params_get_value
http_request_query_params_get_value.argtypes = [c_void_p, c_char_p]
http_request_query_params_get_value.restype = POINTER (String)

# multi-parts
http_request_multi_parts_get = lib.http_request_multi_parts_get
http_request_multi_parts_get.argtypes = [c_void_p, c_char_p]
http_request_multi_parts_get.restype = c_void_p

http_request_multi_parts_get_value = lib.http_request_multi_parts_get_value
http_request_multi_parts_get_value.argtypes = [c_void_p, c_char_p]
http_request_multi_parts_get_value.restype = c_char_p

http_request_multi_parts_get_filename = lib.http_request_multi_parts_get_filename
http_request_multi_parts_get_filename.argtypes = [c_void_p, c_char_p]
http_request_multi_parts_get_filename.restype = c_char_p

http_request_multi_parts_get_saved_filename = lib.http_request_multi_parts_get_saved_filename
http_request_multi_parts_get_saved_filename.argtypes = [c_void_p, c_char_p]
http_request_multi_parts_get_saved_filename.restype = c_char_p

http_request_multi_parts_iter_start = lib.http_request_multi_parts_iter_start
http_request_multi_parts_iter_start.argtypes = [c_void_p]
http_request_multi_parts_iter_start.restype = c_bool

http_request_multi_parts_iter_get_next = lib.http_request_multi_parts_iter_get_next
http_request_multi_parts_iter_get_next.argtypes = [c_void_p]
http_request_multi_parts_iter_get_next.restype = c_void_p

http_request_multi_part_keep_files = lib.http_request_multi_part_keep_files
http_request_multi_part_keep_files.argtypes = [c_void_p]

http_request_multi_part_discard_files = lib.http_request_multi_part_discard_files
http_request_multi_part_discard_files.argtypes = [c_void_p]

http_request_multi_parts_print = lib.http_request_multi_parts_print
http_request_multi_parts_print.argtypes = [c_void_p]

http_request_multi_parts_files_print = lib.http_request_multi_parts_files_print
http_request_multi_parts_files_print.argtypes = [c_void_p]

# body
http_request_body_get_value = lib.http_request_body_get_value
http_request_body_get_value.argtypes = [c_void_p, c_char_p]
http_request_body_get_value.restype = POINTER (String)



from ctypes import CFUNCTYPE, c_void_p, py_object, c_int, c_uint, c_char_p

from ..lib import lib

from .threads import thread_mutex_new, thread_mutex_delete
from .threads import thread_mutex_lock, thread_mutex_unlock

WorkerState = c_int

WorkerMethod = CFUNCTYPE (None, py_object)
WorkerDataDelete = CFUNCTYPE (None, c_void_p)

WORKER_STATE_NONE = 0
WORKER_STATE_AVAILABLE = 1
WORKER_STATE_WORKING = 2
WORKER_STATE_STOPPED = 3
WORKER_STATE_ENDED = 4

worker_state_to_string = lib.worker_state_to_string
worker_state_to_string.argtypes = [WorkerState]
worker_state_to_string.restype = c_char_p

worker_delete = lib.worker_delete
worker_delete.argtypes = [c_void_p]

worker_create = lib.worker_create
worker_create.restype = c_void_p

worker_create_with_id = lib.worker_create_with_id
worker_create_with_id.argtypes = [c_uint]
worker_create_with_id.restype = c_void_p

worker_set_name = lib.worker_set_name
worker_set_name.argtypes = [c_void_p, c_char_p]

worker_get_state = lib.worker_get_state
worker_get_state.argtypes = [c_void_p]
worker_get_state.restype = WorkerState

worker_set_work = lib.worker_set_work
worker_set_work.argtypes = [c_void_p, WorkerMethod]

worker_set_delete_data = lib.worker_set_delete_data
worker_set_delete_data.argtypes = [c_void_p, WorkerMethod]

worker_set_reference = lib.worker_set_reference
worker_set_reference.argtypes = [c_void_p, py_object]

worker_set_remove_reference = lib.worker_set_remove_reference
worker_set_remove_reference.argtypes = [c_void_p, WorkerMethod]

worker_start_with_state = lib.worker_start_with_state
worker_start_with_state.argtypes = [c_void_p, WorkerState]
worker_start_with_state.restype = c_uint

worker_start = lib.worker_start
worker_start.argtypes = [c_void_p]
worker_start.restype = c_uint

worker_resume = lib.worker_resume
worker_resume.argtypes = [c_void_p]
worker_resume.restype = c_uint

worker_stop = lib.worker_stop
worker_stop.argtypes = [c_void_p]
worker_stop.restype = c_uint

worker_end = lib.worker_end
worker_end.argtypes = [c_void_p]
worker_end.restype = c_uint

worker_push_job = lib.worker_push_job
worker_push_job.argtypes = [c_void_p, py_object]
worker_push_job.restype = c_uint

worker_push_job_with_work = lib.worker_push_job_with_work
worker_push_job_with_work.argtypes = [c_void_p, WorkerMethod, py_object]
worker_push_job_with_work.restype = c_uint

@CFUNCTYPE (None, py_object)
def worker_remove_reference (worker):
	worker.queue.pop (0)

class Worker ():
	def __init__ (self, name, work):
		self.worker = worker_create ()
		worker_set_reference (self.worker, self)
		worker_set_name (self.worker, name.encode ('utf-8'))
		worker_set_work (self.worker, work)
		worker_set_remove_reference (self.worker, worker_remove_reference)

		self.queue = []

		self.mutex = thread_mutex_new ()

	def __del__ (self):
		# correctly end internal queue
		self.end ()
		worker_delete (self.worker)

		# remove all saved references
		self.queue.clear ()

		thread_mutex_delete (self.mutex)

	def start (self):
		return worker_start (self.worker)

	def resume (self):
		return worker_resume (self.worker)

	def stop (self):
		return worker_stop (self.worker)

	def push (self, data):
		result = None

		thread_mutex_lock (self.mutex)

		# safely insert object reference into local queue
		self.queue.append (data)

		# process data using internal queue
		result = worker_push_job (self.worker, data)

		thread_mutex_unlock (self.mutex)

		return result

	def end (self):
		return worker_end (self.worker)



from ctypes import c_int, c_void_p

from ..lib import lib

bsem_new = lib.bsem_new
bsem_new.restype = c_void_p

bsem_delete = lib.bsem_delete
bsem_delete.argtypes = [c_void_p]

# inits semaphore to 1 or 0
bsem_init = lib.bsem_init
bsem_init.argtypes = [c_void_p, c_int]

# resets semaphore to 0
bsem_reset = lib.bsem_reset
bsem_reset.argtypes = [c_void_p]

# posts to at least one thread
bsem_post = lib.bsem_post
bsem_post.argtypes = [c_void_p]

# posts to all threads
bsem_post_all = lib.bsem_post_all
bsem_post_all.argtypes = [c_void_p]

# waits on semaphore until semaphore has value 0
bsem_wait = lib.bsem_wait
bsem_wait.argtypes = [c_void_p]



import signal

from ctypes import c_int, c_uint8, c_uint16, c_char_p, c_size_t, c_void_p, c_bool

from .lib import lib

from .version import cerver_version_print_full, pycerver_version_print_full

CERVER_DEFAULT_PORT							= 7000
# CERVER_DEFAULT_PROTOCOL						= PROTOCOL_TCP
CERVER_DEFAULT_USE_IPV6						= False
CERVER_DEFAULT_CONNECTION_QUEUE				= 10

CERVER_DEFAULT_RECEIVE_BUFFER_SIZE			= 4096
# CERVER_DEFAULT_MAX_RECEIVED_PACKET_SIZE		= MAX_UDP_PACKET_SIZE

CERVER_DEFAULT_REUSABLE_FLAGS				= False

CERVER_DEFAULT_POOL_THREADS					= 4

CERVER_DEFAULT_SOCKETS_INIT					= 10

CERVER_DEFAULT_POLL_FDS						= 128
CERVER_DEFAULT_POLL_TIMEOUT					= 2000

CERVER_DEFAULT_MAX_INACTIVE_TIME			= 60
CERVER_DEFAULT_CHECK_INACTIVE_INTERVAL		= 30

CERVER_DEFAULT_AUTH_REQUIRED				= False
CERVER_DEFAULT_MAX_AUTH_TRIES				= 2

CERVER_DEFAULT_ON_HOLD_POLL_FDS				= 64
CERVER_DEFAULT_ON_HOLD_TIMEOUT				= 2000
CERVER_DEFAULT_ON_HOLD_MAX_BAD_PACKETS		= 4
CERVER_DEFAULT_ON_HOLD_CHECK_PACKETS		= False
CERVER_DEFAULT_ON_HOLD_RECEIVE_BUFFER_SIZE	= 4096

CERVER_DEFAULT_USE_SESSIONS					= False

CERVER_DEFAULT_MULTIPLE_HANDLERS			= False

CERVER_DEFAULT_CHECK_PACKETS				= False

CERVER_DEFAULT_UPDATE_TICKS					= 30
CERVER_DEFAULT_UPDATE_INTERVAL_SECS			= 1

CerverType = c_int

CERVER_TYPE_NONE = 0
CERVER_TYPE_CUSTOM = 1
CERVER_TYPE_GAME = 2
CERVER_TYPE_WEB = 3
CERVER_TYPE_FILES = 4

CerverHandlerType = c_int

CERVER_HANDLER_TYPE_NONE = 0
CERVER_HANDLER_TYPE_POLL = 1
CERVER_HANDLER_TYPE_THREADS = 2

# global
cerver_init = lib.cerver_init
cerver_end = lib.cerver_end

# stats
cerver_stats_print = lib.cerver_stats_print
cerver_stats_print.argtypes = [c_void_p, c_bool, c_bool]

# main
cerver_set_alias = lib.cerver_set_alias
cerver_set_alias.argtypes = [c_void_p, c_char_p]

cerver_set_welcome_msg = lib.cerver_set_welcome_msg
cerver_set_welcome_msg.argtypes = [c_void_p, c_char_p]
cerver_set_welcome_msg.restype = c_uint8

cerver_create = lib.cerver_create
cerver_create.argtypes = [CerverType, c_char_p, c_uint16, c_int, c_bool, c_uint16]
cerver_create.restype = c_void_p

cerver_create_web = lib.cerver_create_web
cerver_create_web.argtypes = [c_char_p, c_uint16, c_uint16]
cerver_create_web.restype = c_void_p

def cerver_initialize (end, print_version = True):
	"""
	Function to correctly initialize cerver
	# Parameters
	------------
	### end: func ()
		Function to kill cerver process
	### print_version: bool, optional.
		Flag to choose printing version of cerver.
		Defaults to True.
	"""
	signal.signal (signal.SIGINT, end)
	signal.signal (signal.SIGTERM, end)

	cerver_init ()

	if (print_version):
		cerver_version_print_full ()
		pycerver_version_print_full ()

# configuration
cerver_set_receive_buffer_size = lib.cerver_set_receive_buffer_size
cerver_set_receive_buffer_size.argtypes = [c_void_p, c_size_t]

cerver_set_thpool_n_threads = lib.cerver_set_thpool_n_threads
cerver_set_thpool_n_threads.argtypes = [c_void_p, c_uint16]

cerver_set_handler_type = lib.cerver_set_handler_type
cerver_set_handler_type.argtypes = [c_void_p, CerverHandlerType]

cerver_set_reusable_address_flags = lib.cerver_set_reusable_address_flags
cerver_set_reusable_address_flags.argtypes = [c_void_p, c_bool]

cerver_set_app_handlers = lib.cerver_set_app_handlers
cerver_set_app_handlers.argtypes = [c_void_p, c_void_p, c_void_p]

# start
cerver_start = lib.cerver_start
cerver_start.argtypes = [c_void_p]
cerver_start.restype = c_uint8

# end
cerver_shutdown = lib.cerver_shutdown
cerver_shutdown.argtypes = [c_void_p]
cerver_shutdown.restype = c_uint8

cerver_teardown = lib.cerver_teardown
cerver_teardown.argtypes = [c_void_p]
cerver_teardown.restype = c_uint8



from ctypes import POINTER, CFUNCTYPE, Structure
from ctypes import c_void_p, py_object, c_int, c_uint, c_uint64

from ..lib import lib

JobMethod = CFUNCTYPE (None, c_void_p)
JobDataDelete = CFUNCTYPE (None, c_void_p)

class Job (Structure):
	_fields_ = [
		("method", JobMethod),
		("args", c_void_p),
	]

job_new = lib.job_new
job_new.restype = POINTER (Job)

job_delete = lib.job_delete
job_delete.argtypes = [c_void_p]

job_comparator = lib.job_comparator
job_comparator.argtypes = [c_void_p, c_void_p]
job_comparator.restype = c_int

job_create = lib.job_create
job_create.argtypes = [JobMethod, c_void_p]
job_create.restype = POINTER (Job)

job_get = lib.job_get
job_get.argtypes = [c_void_p]
job_get.restype = POINTER (Job)

job_reset = lib.job_reset
job_reset.argtypes = [c_void_p]

job_return = lib.job_return
job_return.argtypes = [c_void_p, c_void_p]

# JobHandler
job_handler_new = lib.job_handler_new
job_handler_new.restype = c_void_p

job_handler_delete = lib.job_handler_delete
job_handler_delete.argtypes = [c_void_p]

job_handler_create = lib.job_handler_create
job_handler_create.restype = c_void_p

job_handler_get = lib.job_handler_get
job_handler_get.argtypes = [c_void_p]
job_handler_get.restype = c_void_p

job_handler_reset = lib.job_handler_reset
job_handler_reset.argtypes = [c_void_p]

job_handler_signal = lib.job_handler_signal
job_handler_signal.argtypes = [c_void_p]

job_handler_return = lib.job_handler_return
job_handler_return.argtypes = [c_void_p, c_void_p]

job_handler_wait = lib.job_handler_wait
job_handler_wait.argtypes = [c_void_p, py_object, c_void_p]

# JobQueue
JobQueueType = c_int

JOB_QUEUE_TYPE_NONE = 0
JOB_QUEUE_TYPE_JOBS = 1
JOB_QUEUE_TYPE_HANDLERS = 2

JobQueueHandler = CFUNCTYPE (None, py_object)

job_queue_new = lib.job_queue_new
job_queue_new.restype = c_void_p

job_queue_delete = lib.job_queue_delete
job_queue_delete.argtypes = [c_void_p]

job_queue_create = lib.job_queue_create
job_queue_create.argtypes = [JobQueueType]
job_queue_create.restype = c_void_p

job_queue_set_handler = lib.job_queue_set_handler
job_queue_set_handler.argtypes = [c_void_p, JobQueueHandler]

job_queue_push = lib.job_queue_push
job_queue_push.argtypes = [c_void_p, c_void_p]
job_queue_push.restype = c_uint

job_queue_push_job = lib.job_queue_push_job
job_queue_push_job.argtypes = [c_void_p, JobMethod, c_void_p]
job_queue_push_job.restype = c_uint

job_queue_push_job_with_id = lib.job_queue_push_job_with_id
job_queue_push_job_with_id.argtypes = [c_void_p, c_uint64, JobMethod, c_void_p]
job_queue_push_job_with_id.restype = c_uint

job_queue_push_handler = lib.job_queue_push_handler
job_queue_push_handler.argtypes = [c_void_p, c_void_p, c_void_p, c_void_p, JobDataDelete]
job_queue_push_handler.restype = c_uint

job_queue_pull = lib.job_queue_pull
job_queue_pull.argtypes = [c_void_p]
job_queue_pull.restype = c_void_p

job_queue_request = lib.job_queue_request
job_queue_request.argtypes = [c_void_p, c_uint64]
job_queue_request.restype = c_void_p

job_queue_start = lib.job_queue_start
job_queue_start.argtypes = [c_void_p]
job_queue_start.restype = c_uint

job_queue_wait = lib.job_queue_wait
job_queue_wait.argtypes = [c_void_p]

job_queue_signal = lib.job_queue_signal
job_queue_signal.argtypes = [c_void_p]

job_queue_stop = lib.job_queue_stop
job_queue_stop.argtypes = [c_void_p]
job_queue_stop.restype = c_uint

job_queue_clear = lib.job_queue_clear
job_queue_clear.argtypes = [c_void_p]



from ctypes import c_void_p

from ..lib import lib

thread_mutex_new = lib.thread_mutex_new
thread_mutex_new.restype = c_void_p

thread_mutex_delete = lib.thread_mutex_delete
thread_mutex_delete.argtypes = [c_void_p]

thread_mutex_lock = lib.thread_mutex_lock
thread_mutex_lock.argtypes = [c_void_p]

thread_mutex_unlock = lib.thread_mutex_unlock
thread_mutex_unlock.argtypes = [c_void_p]

thread_cond_new = lib.thread_cond_new
thread_cond_new.restype = c_void_p

thread_cond_delete = lib.thread_cond_delete
thread_cond_delete.argtypes = [c_void_p]



from .lib import lib

from .utils.log import LOG_TYPE_NONE, cerver_log_both

PYCERVER_VERSION = "0.8.1".encode ('utf-8')
PYCERVER_VERSION_NAME = "Version 0.8.1".encode ('utf-8')
PYCERVER_VERSION_DATE = "09/08/2021".encode ('utf-8')
PYCERVER_VERSION_TIME = "09:43 CST".encode ('utf-8')
PYCERVER_VERSION_AUTHOR = "Erick Salas".encode ('utf-8')

cerver_version_print_full = lib.cerver_version_print_full
cerver_version_print_version_id = lib.cerver_version_print_version_id
cerver_version_print_version_name = lib.cerver_version_print_version_name

def pycerver_version_print_full ():
	cerver_log_both (
		LOG_TYPE_NONE, LOG_TYPE_NONE,
		"\nPyCerver Version: %s".encode ('utf-8'), PYCERVER_VERSION_NAME
	)

	cerver_log_both (
		LOG_TYPE_NONE, LOG_TYPE_NONE,
		"Release Date & time: %s - %s".encode ('utf-8'),
		PYCERVER_VERSION_DATE, PYCERVER_VERSION_TIME
	)

	cerver_log_both (
		LOG_TYPE_NONE, LOG_TYPE_NONE,
		"Author: %s\n".encode ('utf-8'),
		PYCERVER_VERSION_AUTHOR
	)

def pycerver_version_print_version_id ():
	cerver_log_both (
		LOG_TYPE_NONE, LOG_TYPE_NONE,
		"\nPyCerver Version ID: %s\n".encode ('utf-8'),
		PYCERVER_VERSION
	)

def pycerver_version_print_version_name ():
	cerver_log_both (
		LOG_TYPE_NONE, LOG_TYPE_NONE,
		"\nPyCerver Version: %s\n".encode ('utf-8'),
		PYCERVER_VERSION_NAME
	)



from ctypes import c_void_p, c_char_p, POINTER

from ..lib import lib

from ..types.string import String

http_query_pairs_get_value = lib.http_query_pairs_get_value
http_query_pairs_get_value.argtypes = [c_void_p, c_char_p]
http_query_pairs_get_value.restype = POINTER (String)

http_query_pairs_print = lib.http_query_pairs_print
http_query_pairs_print.argtypes = [c_void_p]


from ctypes import Structure, POINTER, c_int, c_char_p, c_void_p

class HttpOrigin (Structure):
	_fields_ = [
		("len", c_uint),
		("value", c_char_p),
	]

http_origin_new = lib.http_origin_new
http_origin_new.restype = POINTER (HttpOrigin)

http_origin_delete = lib.http_origin_delete
http_origin_delete.argtypes = [c_void_p]

http_origin_get_len = lib.http_origin_get_len
http_origin_get_len.argtypes = [c_void_p]
http_origin_get_len.restype = c_int

http_origin_get_value = lib.http_origin_get_value
http_origin_get_value.argtypes = [c_void_p]
http_origin_get_value.restype = c_char_p

http_origin_print = lib.http_origin_print
http_origin_print.argtypes = [c_void_p]



from ctypes import c_int, c_uint, c_char_p, c_bool, POINTER

from .lib import lib

# main
files_create_dir = lib.files_create_dir
files_create_dir.argtypes = [c_char_p, c_uint]
files_create_dir.restype = c_uint

files_create_recursive_dir = lib.files_create_recursive_dir
files_create_recursive_dir.argtypes = [c_char_p, c_uint]
files_create_recursive_dir.restype = c_uint

files_get_file_extension_reference = lib.files_get_file_extension_reference
files_get_file_extension_reference.argtypes = [c_char_p, POINTER (c_uint)]
files_get_file_extension_reference.restype = c_char_p

file_exists = lib.file_exists
file_exists.argtypes = [c_char_p]
file_exists.restype = c_bool

# images
ImageType = c_int

IMAGE_TYPE_NONE = 0
IMAGE_TYPE_PNG = 1
IMAGE_TYPE_JPEG = 2
IMAGE_TYPE_GIF = 3
IMAGE_TYPE_BMP = 4

files_image_type_to_string = lib.files_image_type_to_string
files_image_type_to_string.argtypes = [ImageType]
files_image_type_to_string.restype = c_char_p

# opens the file and returns the file's image type
files_image_get_type = lib.files_image_get_type
files_image_get_type.argtypes = [c_char_p]
files_image_get_type.restype = ImageType

# returns the correct image type based on the filename's extension
files_image_get_type_by_extension = lib.files_image_get_type_by_extension
files_image_get_type_by_extension.argtypes = [c_char_p]
files_image_get_type_by_extension.restype = ImageType

# returns true if the filename's extension is jpg or jpeg
files_image_extension_is_jpeg = lib.files_image_extension_is_jpeg
files_image_extension_is_jpeg.argtypes = [c_char_p]
files_image_extension_is_jpeg.restype = c_bool

# returns true if the filename's extension is png
files_image_extension_is_png = lib.files_image_extension_is_png
files_image_extension_is_png.argtypes = [c_char_p]
files_image_extension_is_png.restype = c_bool



from ctypes import c_int, c_char_p

from ..lib import lib

http_header = c_int

# Authentication
HTTP_HEADER_AUTHENTICATE = 0
HTTP_HEADER_AUTHORIZATION = 1
HTTP_HEADER_PROXY_AUTHENTICATE = 2
HTTP_HEADER_PROXY_AUTHORIZATION = 3

# Caching
HTTP_HEADER_AGE = 4
HTTP_HEADER_CACHE_CONTROL = 5
HTTP_HEADER_CLEAR_SITE_DATA = 6
HTTP_HEADER_EXPIRES = 7
HTTP_HEADER_WARNING = 8

# Connection management
HTTP_HEADER_CONNECTION = 9
HTTP_HEADER_KEEP_ALIVE = 10

# Content negotiation
HTTP_HEADER_ACCEPT = 11
HTTP_HEADER_ACCEPT_CHARSET = 12
HTTP_HEADER_ACCEPT_ENCODING = 13
HTTP_HEADER_ACCEPT_LANGUAGE = 14

# Controls
HTTP_HEADER_EXPECT = 15

# Cookies
HTTP_HEADER_COOKIE = 16
HTTP_HEADER_SET_COOKIE = 17

# CORS
HTTP_HEADER_ACCESS_CONTROL_ALLOW_ORIGIN = 18
HTTP_HEADER_ACCESS_CONTROL_ALLOW_CREDENTIALS = 19
HTTP_HEADER_ACCESS_CONTROL_ALLOW_HEADERS = 20
HTTP_HEADER_ACCESS_CONTROL_ALLOW_METHODS = 21
HTTP_HEADER_ACCESS_CONTROL_EXPOSE_HEADER = 22
HTTP_HEADER_ACCESS_CONTROL_MAX_AGE = 23
HTTP_HEADER_ACCESS_CONTROL_REQUEST_HEADERS = 24
HTTP_HEADER_ACCESS_CONTROL_REQUEST_METHOD = 25
HTTP_HEADER_ORIGIN = 26
HTTP_HEADER_TIMING_ALLOW_ORIGIN = 27

# Downloads
HTTP_HEADER_CONTENT_DISPOSITION = 28

# Message body information
HTTP_HEADER_CONTENT_LENGTH = 29
HTTP_HEADER_CONTENT_TYPE = 30
HTTP_HEADER_CONTENT_ENCODING = 31
HTTP_HEADER_CONTENT_LANGUAGE = 32
HTTP_HEADER_CONTENT_LOCATION = 33

# Proxies
HTTP_HEADER_FORWARDED = 34
HTTP_HEADER_VIA = 35

# Redirects
HTTP_HEADER_LOCATION = 36

# Request context
HTTP_HEADER_FROM = 37
HTTP_HEADER_HOST = 38
HTTP_HEADER_REFERER = 39
HTTP_HEADER_REFERRER_POLICY = 40
HTTP_HEADER_USER_AGENT = 41

# Response context
HTTP_HEADER_ALLOW = 42
HTTP_HEADER_SERVER = 43

# Range requests
HTTP_HEADER_ACCEPT_RANGES = 44
HTTP_HEADER_RANGE = 45
HTTP_HEADER_IF_RANGE = 46
HTTP_HEADER_CONTENT_RANGE = 47

# WebSockets
HTTP_HEADER_SEC_WEBSOCKET_KEY = 48
HTTP_HEADER_SEC_WEBSOCKET_EXTENSIONS = 49
HTTP_HEADER_SEC_WEBSOCKET_ACCEPT = 50
HTTP_HEADER_SEC_WEBSOCKET_PROTOCOL = 51
HTTP_HEADER_SEC_WEBSOCKET_VERSION = 52

# Other
HTTP_HEADER_DATE = 53
HTTP_HEADER_UPGRADE = 54

HTTP_HEADER_INVALID = 53

http_header_string = lib.http_header_string
http_header_string.argtypes = [http_header]
http_header_string.restype = c_char_p

http_header_description = lib.http_header_description
http_header_description.argtypes = [http_header]
http_header_description.restype = c_char_p

http_header_type_by_string = lib.http_header_type_by_string
http_header_type_by_string.argtypes = [c_char_p]
http_header_type_by_string.restype = http_header



from ctypes import c_int, c_char_p, c_bool

from ..lib import lib

ContentType = c_int

HTTP_CONTENT_TYPE_NONE = 0
HTTP_CONTENT_TYPE_AAC = 1
HTTP_CONTENT_TYPE_ABIWORD = 2
HTTP_CONTENT_TYPE_ARCHIVE = 3
HTTP_CONTENT_TYPE_AVI = 4
HTTP_CONTENT_TYPE_KINDLE = 5
HTTP_CONTENT_TYPE_BIN = 6
HTTP_CONTENT_TYPE_BMP = 7
HTTP_CONTENT_TYPE_BZIP = 8
HTTP_CONTENT_TYPE_BZIP2 = 9
HTTP_CONTENT_TYPE_CSHELL = 10
HTTP_CONTENT_TYPE_CSS = 11
HTTP_CONTENT_TYPE_CSV = 12
HTTP_CONTENT_TYPE_DOC = 13
HTTP_CONTENT_TYPE_DOCX = 14
HTTP_CONTENT_TYPE_EOT = 15
HTTP_CONTENT_TYPE_EPUB = 16
HTTP_CONTENT_TYPE_GZIP = 17
HTTP_CONTENT_TYPE_GIF = 18
HTTP_CONTENT_TYPE_HTML = 19
HTTP_CONTENT_TYPE_ICO = 20
HTTP_CONTENT_TYPE_ICS = 21
HTTP_CONTENT_TYPE_JAR = 22
HTTP_CONTENT_TYPE_JPG = 23
HTTP_CONTENT_TYPE_JPEG = 24
HTTP_CONTENT_TYPE_JS = 25
HTTP_CONTENT_TYPE_JSON = 26
HTTP_CONTENT_TYPE_JSONLD = 27
HTTP_CONTENT_TYPE_MIDI = 28
HTTP_CONTENT_TYPE_MJS = 29
HTTP_CONTENT_TYPE_MP3 = 30
HTTP_CONTENT_TYPE_CDA = 31
HTTP_CONTENT_TYPE_MP4 = 32
HTTP_CONTENT_TYPE_MPEG = 33
HTTP_CONTENT_TYPE_MPKG = 34
HTTP_CONTENT_TYPE_ODP = 35
HTTP_CONTENT_TYPE_ODS = 36
HTTP_CONTENT_TYPE_ODT = 37
HTTP_CONTENT_TYPE_OGA = 38
HTTP_CONTENT_TYPE_OGV = 39
HTTP_CONTENT_TYPE_OGX = 40
HTTP_CONTENT_TYPE_OPUS = 41
HTTP_CONTENT_TYPE_OTF = 42
HTTP_CONTENT_TYPE_PNG = 43
HTTP_CONTENT_TYPE_PDF = 44
HTTP_CONTENT_TYPE_PHP = 45
HTTP_CONTENT_TYPE_PPT = 46
HTTP_CONTENT_TYPE_PPTX = 47
HTTP_CONTENT_TYPE_RAR = 48
HTTP_CONTENT_TYPE_RTF = 49
HTTP_CONTENT_TYPE_SHELL = 50
HTTP_CONTENT_TYPE_SVG = 51
HTTP_CONTENT_TYPE_SWF = 52
HTTP_CONTENT_TYPE_TAR = 53
HTTP_CONTENT_TYPE_TIFF = 54
HTTP_CONTENT_TYPE_TS = 55
HTTP_CONTENT_TYPE_TTF = 56
HTTP_CONTENT_TYPE_TXT = 57
HTTP_CONTENT_TYPE_VSD = 58
HTTP_CONTENT_TYPE_WAV = 59
HTTP_CONTENT_TYPE_WEBA = 60
HTTP_CONTENT_TYPE_WEBM = 61
HTTP_CONTENT_TYPE_WEBP = 62
HTTP_CONTENT_TYPE_WOFF = 63
HTTP_CONTENT_TYPE_WOFF2 = 64
HTTP_CONTENT_TYPE_XHTML = 65
HTTP_CONTENT_TYPE_XLS = 66
HTTP_CONTENT_TYPE_XLSX = 67
HTTP_CONTENT_TYPE_XML = 68
HTTP_CONTENT_TYPE_XUL = 69
HTTP_CONTENT_TYPE_ZIP = 70
HTTP_CONTENT_TYPE_3GP = 71
HTTP_CONTENT_TYPE_3G2 = 72
HTTP_CONTENT_TYPE_7Z = 73

http_content_type_extension = lib.http_content_type_extension
http_content_type_extension.argtypes = [ContentType]
http_content_type_extension.restype = c_char_p

http_content_type_description = lib.http_content_type_description
http_content_type_description.argtypes = [ContentType]
http_content_type_description.restype = c_char_p

http_content_type_mime = lib.http_content_type_mime
http_content_type_mime.argtypes = [ContentType]
http_content_type_mime.restype = c_char_p

http_content_type_by_mime = lib.http_content_type_by_mime
http_content_type_by_mime.argtypes = [c_char_p]
http_content_type_by_mime.restype = ContentType

http_content_type_mime_by_extension = lib.http_content_type_mime_by_extension
http_content_type_mime_by_extension.argtypes = [c_char_p]
http_content_type_mime_by_extension.restype = c_char_p

http_content_type_is_json = lib.http_content_type_is_json
http_content_type_is_json.argtypes = [c_char_p]
http_content_type_is_json.restype = c_bool



from ctypes import c_int, c_uint, c_char_p, c_bool

from ..lib import lib

LogType = c_int

LOG_TYPE_NONE = 0
LOG_TYPE_ERROR = 1
LOG_TYPE_WARNING = 2
LOG_TYPE_SUCCESS = 3
LOG_TYPE_DEBUG = 4
LOG_TYPE_TEST = 5
LOG_TYPE_CERVER = 6
LOG_TYPE_CLIENT = 7
LOG_TYPE_CONNECTION = 8
LOG_TYPE_HANDLER = 9
LOG_TYPE_ADMIN = 10
LOG_TYPE_EVENT = 11
LOG_TYPE_PACKET = 12
LOG_TYPE_REQ = 13
LOG_TYPE_FILE = 14
LOG_TYPE_HTTP = 15
LOG_TYPE_GAME = 16
LOG_TYPE_PLAYER = 17

# config
LogOutputType = c_int

LOG_OUTPUT_TYPE_NONE = 0
LOG_OUTPUT_TYPE_STD	 = 1
LOG_OUTPUT_TYPE_FILE = 2
LOG_OUTPUT_TYPE_BOTH = 3

cerver_log_get_output_type = lib.cerver_log_get_output_type
cerver_log_get_output_type.restype = LogOutputType

cerver_log_set_output_type = lib.cerver_log_set_output_type
cerver_log_set_output_type.argtypes = [LogOutputType]

cerver_log_set_path = lib.cerver_log_set_path
cerver_log_set_path.argtypes = [c_char_p]
cerver_log_set_path.restype = c_uint

cerver_log_set_update_interval = lib.cerver_log_set_update_interval
cerver_log_set_update_interval.argtypes = [c_uint]

LogTimeType = c_int

LOG_TIME_TYPE_NONE = 0
LOG_TIME_TYPE_TIME = 1
LOG_TIME_TYPE_DATE = 2
LOG_TIME_TYPE_BOTH = 3

cerver_log_time_type_to_string = lib.cerver_log_time_type_to_string
cerver_log_time_type_to_string.argtypes = [LogTimeType]
cerver_log_time_type_to_string.restype = c_char_p

cerver_log_time_type_description = lib.cerver_log_time_type_description
cerver_log_time_type_description.argtypes = [LogTimeType]
cerver_log_time_type_description.restype = c_char_p

cerver_log_get_time_config = lib.cerver_log_get_time_config
cerver_log_get_time_config.restype = LogTimeType

cerver_log_set_time_config = lib.cerver_log_set_time_config
cerver_log_set_time_config.argtypes = [LogTimeType]

cerver_log_set_local_time = lib.cerver_log_set_local_time
cerver_log_set_local_time.argtypes = [c_bool]

cerver_log_set_quiet = lib.cerver_log_set_quiet
cerver_log_set_quiet.argtypes = [c_bool]

# public
cerver_log = lib.cerver_log
cerver_log_with_date = lib.cerver_log_with_date
cerver_log_both = lib.cerver_log_both
cerver_log_msg = lib.cerver_log_msg
cerver_log_error = lib.cerver_log_error
cerver_log_warning = lib.cerver_log_warning
cerver_log_success = lib.cerver_log_success
cerver_log_debug = lib.cerver_log_debug
cerver_log_raw = lib.cerver_log_raw
cerver_log_line_break = lib.cerver_log_line_break
